/*Copyright (C) 2010-2022 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEF_FIGHT_SCENE
#define DEF_FIGHT_SCENE

#include "Engines/Resources/TextureManager.hpp"
#include "Engines/Sound/SoundEngine.hpp"
#include "Engines/Physics/PhysicsWorld.hpp"

#include "Player/Player.hpp"
#include "Player/AI/AI.hpp"
#include "Player/Human/Human.hpp"
#include "Player/Human/PlayerKeys.hpp"

#include "Arena/Arena.hpp"
#include "Weapon/WeaponMedium.hpp"

#include "FightCameraHandler.hpp"

#include <string>
#include <vector>

enum class FightMode {Default, Versus};

class FightScene
{
    public:
        FightScene();
        FightScene(const FightScene &copied) = delete;
        FightScene &operator=(const FightScene &copied) = delete;
        ~FightScene();
        
        void setMode(FightMode mode);
        void setArena(const std::string &arenaPath);
        
        void addAIPlayer(const std::string &name);
        void addHumanPlayer(const std::string &name, int deviceID, const PlayerKeys &keys);
        
        void initialize(TextureManager &textureManager, SoundEngine &soundEngine, bool showHitboxes);
        
        void processKeyEvent(const SDL_Event &event); ///For human-controlled players
        void updateKeyState(const SDL_Event &event); ///For human-controlled players
        void update(float frameTime);
        
        void draw();
        
        bool isFinished(int &winnerIxDst, std::string &winnerNameDst) const;
        
        std::vector<const Player*> getPlayers() const;
        FightCameraHandler getCameraHandler(FightCameraHandler::CameraMode cameraMode) const;
        float getMinCameraWidth() const;
        
    private:
        ///Fight scene parameters
        FightMode m_mode;
        std::string m_arenaPath;
        bool m_showHitboxes;
        
        ///Fight scene objects
        PhysicsWorld m_physicsWorld;
        Arena m_arena;
        
        std::vector<Player*> m_players;
        std::list<Human> m_humansList;
        std::list<AI> m_aisList;
        
        WeaponMedium *m_weapons;
};

#endif
