/*Copyright (C) 2010-2022 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Directory.hpp"

namespace Directory
{
    void create(const std::string &path)
    {
        #if defined(__unix__) || defined(__APPLE__)
            mkdir(path.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH); //chmod 0755
        
        #elif defined(__HAIKU__)
            create_directory(path.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
        
        #elif defined(_WIN32)
            CreateDirectory(path.c_str(), nullptr);
        
        #endif
    }

    std::string getHNMHome()
    {
        std::string homePath;
        
        #if defined(__unix__) || defined(__APPLE__)
            homePath = std::string(getenv("HOME"));
            homePath += "/.hikounomizu/";
        
        #elif defined(__HAIKU__)
            BPath path;
            find_directory(B_USER_SETTINGS_DIRECTORY, &path);
            
            homePath = std::string(path.Path());
            homePath += "/HikouNoMizu/";
        
        #elif defined(_WIN32)
            TCHAR szPath[MAX_PATH];
            if (SUCCEEDED(SHGetFolderPath(nullptr, CSIDL_APPDATA, nullptr, 0, szPath)))
            {
                homePath = std::string(szPath);
                homePath += "\\hikounomizu\\";
            }
        
        #endif
        
        return homePath;
    }
}
