/*Copyright (C) 2010-2022 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Format.hpp"

Format::Format(Emphasis emph) :
m_mode(Format::ColorMode::None), m_emphasis(emph)
{
    generateANSI();
}

Format::Format(const Color &fg, Emphasis emph) :
m_color1(fg), m_mode(Format::ColorMode::Foreground), m_emphasis(emph)
{
    generateANSI();
}

Format::Format(const Color &fg, const Color &bg, Emphasis emph) :
m_color1(fg), m_color2(bg), m_mode(Format::ColorMode::FgAndBg), m_emphasis(emph)
{
    generateANSI();
}

void Format::generateANSI()
{
    std::string emphasis = "";
    if (m_emphasis == Format::Emphasis::Bold) emphasis = "\x1b[1m";
    else if (m_emphasis == Format::Emphasis::Faint) emphasis = "\x1b[2m";
    else if (m_emphasis == Format::Emphasis::Italic) emphasis = "\x1b[3m";
    
    if (m_mode == Format::ColorMode::None)
    {
        m_ansiCodes = emphasis;
    }
    else if (m_mode == Format::ColorMode::Foreground)
    {
        m_ansiCodes = hnm::format("{}\x1b[38;2;{};{};{}m",
                            emphasis, static_cast<short>(m_color1.red),
                                      static_cast<short>(m_color1.green),
                                      static_cast<short>(m_color1.blue));
    }
    else
    {
        m_ansiCodes = hnm::format("{}\x1b[38;2;{};{};{}m\x1b[48;2;{};{};{}m",
                            emphasis,
                            static_cast<short>(m_color1.red),
                            static_cast<short>(m_color1.green),
                            static_cast<short>(m_color1.blue),
                            static_cast<short>(m_color2.red),
                            static_cast<short>(m_color2.green),
                            static_cast<short>(m_color2.blue));
    }
}

std::string Format::apply(const std::string &str) const
{
    return m_ansiCodes + str + "\x1b[0m";
}
