/*Copyright (C) 2010-2022 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEF_FORMAT
#define DEF_FORMAT

#include <iostream>
#include <sstream>
#include <vector>
#include <string>
#include "Structs/Color.hpp"

class Format
{
    public:
        enum class Emphasis {None, Bold, Faint, Italic};
        
        Format(Emphasis emph);
        Format(const Color &color, Emphasis emph=Emphasis::None);
        Format(const Color &fg, const Color &bg, Emphasis emph=Emphasis::None);
        
        std::string apply(const std::string &str) const;
    
    private:
        enum class ColorMode {None, Foreground, FgAndBg};
        
        void generateANSI();
        std::string m_ansiCodes;
        
        Color m_color1, m_color2;
        ColorMode m_mode;
        Emphasis m_emphasis;
};

namespace hnm
{
    namespace
    {
        ///Converts arbitrarily typed arguments to std::string if possible
        ///and adds it to a resulting std::vector<std::string>
        template <typename...>
        void addToVect(std::vector<std::string> &/*target*/) {}
        
        template <typename T, typename... Ts>
        void addToVect(std::vector<std::string> &target, T&& head, Ts&&... tail)
        {
            std::stringstream sstream;
            sstream << head;
            target.push_back(sstream.str());
            
            addToVect(target, tail...);
        }
    }
    
    template <typename... Ts>
    inline std::string format(const std::string &sentence, Ts&&... args)
    {
        std::vector<std::string> strArgs;
        addToVect(strArgs, args...);
        
        std::string formatted(sentence);
        
        unsigned int argsIndex = 0;
        std::size_t findIndex = 0;
        bool browsed = false;
        while (!browsed)
        {
            if (argsIndex >= strArgs.size())
                browsed = true;
            else
            {
                findIndex = formatted.find("{}", findIndex);
                if (findIndex == std::string::npos)
                    browsed = true;
                else
                {
                    formatted.replace(findIndex, 2, strArgs[argsIndex]);
                    findIndex += strArgs[argsIndex].length();
                    argsIndex++;
                }
            }
        }
        
        return formatted;
    }
    
    inline void print(const std::string &str)
    {
        std::cout << str << std::endl;
    }
    
    inline void print(const Format &style, const std::string &str)
    {
        std::cout << style.apply(str) << std::endl;
    }
    
    inline void printerr(const std::string &str)
    {
        std::cerr << Format(Color(255, 0, 0)).apply(str) << std::endl;
    }
    
};

#endif
